use strict;
use warnings;
use JSON;
use File::Basename;
use Digest::SHA qw(hmac_sha256_hex);
use constant NL => $^O eq 'MSWin32' ? "\015\012" : "\012";


$| = 1;

if(scalar @ARGV != 3)
{
	print "Usage: " , basename(__FILE__), " device_id seed SecurePreferences ", NL;
	exit 1;
}

my ($device_id, $seed, $sec_pref_file) = @ARGV;

#     Secure Preferences
open my $fh, '<', $sec_pref_file or die $!;
undef $/;
my $json_raw_data = <$fh>;
close $fh;

# ,    :
#  Secure Preferences  <    \u003C,
#   JSON  ,    HMAC   as-is 
$json_raw_data =~ s/\\u/\\\\u/g;

my $json = JSON->new->allow_nonref;
my $sec_pref = $json->decode($json_raw_data);

my $reference_hmacs = $sec_pref->{protection}->{macs}->{extensions}->{settings};

#         HMAC
my %ext_settings = %{$sec_pref->{extensions}->{settings}};
# seed    
my $key = pack 'H*', $seed;

print NL;
while(my ($ext_name, $ext_prefs) = each %ext_settings)
{
	#  JSON- 
	remove_empty($ext_prefs);

	my $extension_path = 'extensions.settings.' . $ext_name;
	my $extension_settings = $json->canonical->encode($ext_prefs);
	$extension_settings =~ s/\\\\u/\\u/g;
	my $message = $device_id . $extension_path . $extension_settings;

	my $hmac = uc hmac_sha256_hex($message, $key);

	#      HMAC'
	#      4 ,     
	my $ref_hmac = $reference_hmacs->{$ext_name};

	my $our_hmac_part = substr($hmac, 0, 4) . '...' . substr($hmac, -4);
	my $ref_hmac_part = substr($ref_hmac, 0, 4) . '...' . substr($ref_hmac, -4);

	print $ext_name, ' - our: ', $our_hmac_part, '; ref: ', $ref_hmac_part, '; ', ($hmac eq $ref_hmac ? 'OK' : 'ERR'), NL;
}

#  super_ma
my $json_macs = $sec_pref->{protection}->{macs};
my $raw_macs = $json->canonical->encode($json_macs);

my $message = $device_id . $raw_macs;

my $hmac = uc hmac_sha256_hex($message, $key);
my $ref_hmac = $sec_pref->{protection}->{super_mac};

my $our_hmac_part = substr($hmac, 0, 4) . '...' . substr($hmac, -4);
my $ref_hmac_part = substr($ref_hmac, 0, 4) . '...' . substr($ref_hmac, -4);

print NL, 'Super mac - our: ', $our_hmac_part, '; ref: ', $ref_hmac_part, '; ', ($hmac eq $ref_hmac ? 'OK' : 'ERR'), NL;


sub remove_empty
{
	my $hash_ref = shift;

	my @queue = ($hash_ref);
	while(my $ref = shift @queue)
	{
		next unless ref $ref eq ref {};

		foreach my $key(keys %$ref)
		{
			if(ref $ref->{$key} eq ref {})
			{
				if(scalar values $ref->{$key} == 0)
				{
					push @queue, $hash_ref;
					delete $ref->{$key};
					next;
				}

				push @queue, $ref->{$key};
			}
			elsif(ref $ref->{$key} eq ref [])
			{
				if(@{$ref->{$key}} == 0)
				{
					push @queue, $hash_ref;
					delete $ref->{$key} ;
				}
			}
		}
	}
}
