use strict;
use warnings;
use File::Basename;
use Fcntl ':seek';
use constant NL => $^O eq 'MSWin32' ? "\015\012" : "\012";

$| = 1;

if(scalar @ARGV != 1)
{
	print "Usage: " , basename(__FILE__), " resources.pak", NL;
	exit 1;
}

open my $fh, '<', $ARGV[0] or die $!;

# Читаем заголовок resources.pak и извлекаем информацию о версии, кол-ве ресурсов и кодировке
# 4 byte version number + 4 byte number of resources + 1 byte encoding
read $fh, my $header, 9;

my ($version, $resources_total, $encoding) = unpack 'VVC', $header;

print "Version: $version", NL, "Resources: $resources_total", NL, "Encoding: ", $encoding, NL, NL;

# Перечисляем ресурсы и их смещения в файле
# 2 byte resource id  + 4 byte resource offset in file

# Учитываем ресурс с ID 0, означающий конец списка
my ($last_id, $last_offset) = (0, 0);
for(my $i = 0; $i < $resources_total + 1; $i++)
{
	read $fh, $header, 6;
	my ($id, $offset) = unpack 'SV', $header;

	if($last_id)
	{
		my $size = $offset - $last_offset;
		# Делаем предположение, что seed всегда состоит из 64 символов
		if($size == 64)
		{
			print "ID: $last_id", NL, "Offset: $last_offset", NL, "Size: $size", NL;

			seek $fh, $last_offset, SEEK_SET;
			read $fh, my $data, 64;

			print "Resource HEX data: ", unpack('H*', $data), NL;

			exit 0;
		}
	}

	$last_id = $id;
	$last_offset = $offset;

	if($id == 0)
	{
		last;
	}
}
